#ifndef DOS_DOS_H
#define DOS_DOS_H
/*
**    $VER: dos.h 53.21 (29.9.2013)
**
**    Standard C header for AmigaDOS
**
**    Copyright (C) 1985-2009 Hyperion Entertainment VOF and Amiga, Inc.
**        All Rights Reserved
**
**    Copyright (c) 2010 Hyperion Entertainment CVBA.
**        All Rights Reserved.
*/ 

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif
#ifndef EXEC_PORTS_H
#include <exec/ports.h>
#endif
#ifndef DOS_ERRORS_H
#include <dos/errors.h>
#endif

/****************************************************************************/

#ifdef __cplusplus
#ifdef __USE_AMIGAOS_NAMESPACE__
namespace AmigaOS {
#endif
extern "C" {
#endif

/****************************************************************************/

#ifdef __GNUC__
   #ifdef __PPC__
    #pragma pack(2)
   #endif
#elif defined(__VBCC__)
   #pragma amiga-align
#endif

/****************************************************************************/

#define DOSNAME "dos.library"

/****************************************************************************/
/* These are the return codes used by convention by AmigaDOS commands.      */
/* See FAILAT and IF commands for relevance to EXECUTE files.               */
/* See RunCommand() and CreateNewProc() for relevance to all DOS processes. */

#define RETURN_OK      0  /* No problems, success */
#define RETURN_WARN    5  /* A warning only, ie; for the shell 'IF' command */
#define RETURN_ERROR  10  /* Something caused a normal sort of error */
#define RETURN_FAIL   20  /* Severe failure, ie; missing system library */


/****************************************************************************/
/* Predefined Amiga DOS global constants */

#define DOSTRUE        (-1L)
#define DOSFALSE       (0L)

#define BITSPERBYTE     8
#define BYTESPERLONG    4
#define BITSPERLONG     32
#define MAXINT32        0x7FFFFFFF
#define MININT32        0x80000000


/****************************************************************************/
/* All BCPL data must be long word aligned.  BCPL pointers are the long word
 * address (i.e byte address divided by 4 (>>2)) 
 */

typedef int32 BPTR; /* Long word (BCPL) pointer */
typedef int32 BSTR; /* Long word pointer to BCPL string */

/* BCPL style 'NULL' */
#define ZERO         ((BPTR)0)

/* Convert BPTR to typical C pointer */
#define BADDR(x)     ((APTR)((uint32)(x) << 2))

/* Convert address into a BPTR */
#define MKBADDR(x)   ((BPTR)(((uint32)(x)) >> 2))

/*
 * README -- BCPL STRINGS:  
 *
 * Original BCPL strings had a length in the first byte and then
 * the actual characters following.  
 * For example:  s[0]=3  s[1]='S'  s[2]='Y'  s[3]='S'
 *
 * From V48+, BCPL strings MUST also have a '\0' nul-terminator
 * byte on the end, for safe interoperability with C-strings.
 * The additional nul-terminator byte is not counted in the length,
 * neither is the length byte itself.
 * For example:  s[0]=3  s[1]='S'  s[2]='Y'  s[3]='S'  s[4]=0
 *
 * DOS library now provides functions to handle these correctly.
 *     IDOS->CopyStringCToBSTR() and IDOS->CopyStringBSTRToC(). 
 */

/****************************************************************************/
/* Mode parameter to Open(), all files are positioned at beginning of file. */

#define MODE_OLDFILE        1005    /* Open existing file for read/write,
                                       fails if file doesn't exist. */

#define MODE_NEWFILE        1006    /* Open freshly created file, read/write,
                                       deletes an old file, exclusive lock. */

#define MODE_READWRITE      1004    /* Open old file, shared lock, read/write,
                                       creates file if it doesn't exist. */

/****************************************************************************/
/* Relative position to Seek(deprecated) and ChangeFilePosition()  */

#define OFFSET_BEGINNING     -1     /* relative to Begining Of File */
#define OFFSET_CURRENT        0     /* relative to Current file position */
#define OFFSET_END            1     /* relative to End Of File      */


/****************************************************************************/
/* Passed as 'accessMode' to Lock() */

#define SHARED_LOCK     -2    /* Object is accessible by all others */
#define ACCESS_READ     -2    /* Somewhat misleading old synonym */

#define EXCLUSIVE_LOCK  -1    /* No other access is allowed */
#define ACCESS_WRITE    -1    /* Somewhat misleading old synonym */

/****************************************************************************/
/* Flags for the bitmasks returned by LockTags() when using LK_InfoMask tag.*/
/* These provide detailed information on the string being resolved.  V53.57 */

#define  LOCKB_SOFTLINK_MIDPART   8  
  /* One or more soft links were found in the middle of the path */
  /* but none of these formed the last component of the path.    */

#define  LOCKB_SOFTLINK_ENDPART   9
  /* One soft link exists as the last component of the path. */
 
#define  LOCKB_DEVICE            16
  /* The supplied path is a device relative specification. */
 
#define  LOCKB_VOLUME            17
  /* The supplied path is a volume relative specification. */
 
#define  LOCKB_ASSIGNMENT        18
  /* The supplied path is an assignment relative specification. */
 
#define  LOCKB_PROGDIR           19
  /* The supplied path is a PROGDIR: relative specification. */
 
#define  LOCKB_CURRDIR           20
  /* The supplied path is a CURRDIR: relative specification. */

#define  LOCKB_ROOTDIR           21
  /* The supplied path is a root directory relative specification. */

#define  LOCKB_FILESYS           24
  /* The supplied path returned the root of the default filesystem. */

/* Field definitions of the above. */
#define  LOCKF_SOFTLINK_MIDPART    (1L<<LOCKB_SOFTLINK_MIDPART)  
#define  LOCKF_SOFTLINK_ENDPART    (1L<<LOCKB_SOFTLINK_ENDPART)
#define  LOCKF_DEVICE              (1L<<LOCKB_DEVICE)
#define  LOCKF_VOLUME              (1L<<LOCKB_VOLUME)
#define  LOCKF_ASSIGNMENT          (1L<<LOCKB_ASSIGNMENT)
#define  LOCKF_PROGDIR             (1L<<LOCKB_PROGDIR)
#define  LOCKF_CURRDIR             (1L<<LOCKB_CURRDIR)
#define  LOCKF_ROOTDIR             (1L<<LOCKB_ROOTDIR)
#define  LOCKF_FILESYS             (1L<<LOCKB_FILESYS)

/****************************************************************************/
/* Result flags for WaitForData(). */

#define WFDB_READ     0  /* Stream has data ready to be read */
#define WFDB_WRITE    1  /* Stream is ready to accept more data to be written.*/

#define WFDF_READ    (1<<WFDB_READ)
#define WFDF_WRITE   (1<<WFDB_WRITE)

/****************************************************************************/
/* NotifyDosListChange(), NotifyProcListChange(), termination signal value. */

#define NPLC_END     0xFFFFFFFFUL
#define NDLC_END     0xFFFFFFFFUL

/****************************************************************************/
/* SetBlockingMode() mode values which define whether read/write access to a
   stream will block until a request is satisfied or will return immediately.*/

#define SBM_BLOCKING        1
#define SBM_NON_BLOCKING    2

/****************************************************************************/
/* GetPID() function, this flag determins which PID is returned.     53.58  */

#define   GPID_PROCESS      1
#define   GPID_PARENT       2

/****************************************************************************/
/* Length constants specifically for new vector-port based filesystems.     */
/* Older filesystem designs may not be able to reach these lengths,         */
/* so use IDOS->FileSystemAttr() to find out filesystem specific values.    */

#ifndef   MAX_VP_FILENAME
#define   MAX_VP_FILENAME   255
#endif

#ifndef   MAX_VP_COMMENT
#define   MAX_VP_COMMENT    255
#endif

#ifndef   MAX_VP_SOFTLINK_TARGET
#define   MAX_VP_SOFTLINK_TARGET   4000
#endif

/****************************************************************************/

struct DateStamp
{
    int32   ds_Days;     /* Number of days since Jan. 1, 1978 */
    int32   ds_Minute;   /* Number of minutes past midnight */
    int32   ds_Tick;     /* Number of ticks past minute */
}; 
/* DateStamp */

#define TICKS_PER_SECOND 50    /* Fifty ticks = one second  */
                               /* One tick = twenty milliseconds */

/****************************************************************************/
/* V52 Struct ExamineData, returned by ExamineObject() and ExamineDir().
 * This structure supports unlimited string lengths, 64 bit file sizes,
 * automatic resolution of link targets.
 *
 * NOTE; Applications DO NOT allocate these, the filesystem does it.
 *       Also, the filesystem itself must always allocate these using
 *       AllocDosObject(), otherwise they will simply not work.
 */ 

struct  ExamineData
{ 
    struct MinNode   EXDnode;      /* MinList node.                           */
    uint32           EXDinfo;      /* Extra list data, general purpose use.   */

    /* These are private members exclusively for the FS & DOS.                */
    uint32           FSPrivate;    /* FILESYSTEM ONLY - Private usage.        */
    uint32           Reserved0;    /* DOS RESERVED - Private usage.           */
    APTR             DOSPrivate;   /* DOSLIB ONLY - Private usage.            */
                                   /*                                         */
    /* The following are public fields.                                       */
    uint32           StructSize;   /* (RO) Full size of the structure.        */
    uint32           Type;         /* Entry type; use EXD_IS_ macros below.   */
    int64            FileSize;     /* Size of a file, otherwise set to -1LL.  */
    struct DateStamp Date;         /* Days, Mins, Ticks.                      */
    uint32           UseCount;     /* Filesystem linked object count.[Note 1] */
    uint64           ObjectID;     /* Unique object identifier.      [Note 2] */
    STRPTR           Name;         /* (RO) Nul-terminated object name.        */
    uint32           NameSize;     /* (RO) Allocated size of the name buffer. */
    STRPTR           Comment;      /* (RO) Nul-terminated comment string.     */
    uint32           CommentSize;  /* (RO) Allocated size of comment buffer.  */
    STRPTR           Link;         /* (RO) Nul-terminated target name string. */
    uint32           LinkSize;     /* (RO) Allocated size of target buffer.   */
    uint32           Protection;   /* Protection status bitmask.              */
    uint32           OwnerUID;     /* Owner info. [Note 3]                    */
    uint32           OwnerGID;     /* Group info. [Note 3]                    */
    uint32           Reserved2;    /* DOS reserved field.                     */
    uint32           Reserved3;    /* DOS reserved field.                     */
    uint32           Reserved4;    /* DOS reserved field.                     */
};
/*	(RO) = Read Only, these fields must not be modified by the application.   */
/* ExamineData */

/*
 * [Note 1] This field may or may not be supported by individual filesystems.
 *          Supporting filesystems use this field to indicate the reference
 *          count for linked objects, a value of one indicates that the
 *          object has no additional references.
 *          This field will generally be zero for the DOS fallback code,
 *          or for filesystems that do not support this field directly,
 *          it will be undefined if you fail to specify the EXF_USECOUNT
 *          bit in the EX_DataFields mask.
 *
 * [Note 2] This field is the unique object identfier for all objects on
 *          this volume, the value used here may be any arbitrary value
 *          as long as it is unique from all other objects on this volume,
 *          it may use the object node address, the disk header block number
 *          or other such values.
 *          This field will be zero for filesystems that do not support
 *          this field directly, it will be undefined if you fail to specify
 *          the EXF_OBJECTID bit in the EX_DataFields mask.
 *
 * [Note 3] Historically, these fields only used the low 16 bits.
 *          For extensibility, the size of these are now 32 bit fields.
 *          However, for interoperability, it is recommended that values
 *          greater than 65535 be avoided where possible so reading only
 *          the low 16 bits results in the same value for legacy applications.
 *          These fields will be undefined if you fail to specify the
 *          EXF_OWNER bit in the EX_DataFields mask.
 *          (See extended information relating to the UID/GID fields below.)
 */

/****************************************************************************/
/* Bit mask field values that are passed in the EX_DataFields tag,
 * this determines what data is returned from the ExamineDir() function.
 *
 * The data value returned in the respective fields will be undefined if
 * the mask bit is not specified, NEVER assume otherwise.
 */

#define  EXF_DATE           (1<<0)
#define  EXF_USECOUNT       (1<<1)
#define  EXF_NAME           (1<<2)
#define  EXF_COMMENT        (1<<3)
#define  EXF_LINK           (1<<4)
#define  EXF_PROTECTION     (1<<5)
#define  EXF_OWNER          (1<<6)   /* both Group and User ID */
#define  EXF_TYPE           (1<<7)   /* MUST be specified for EXD_IS macros */
#define  EXF_SIZE           (1<<8)
#define  EXF_OBJECTID       (1<<9)

#define  EXF_spare1         (1<<10)  /* reserved for future EXF_ALL feature */
#define  EXF_spare2         (1<<11)  /* reserved for future EXF_ALL feature */
#define  EXF_spare3         (1<<12)  /* reserved for future EXF_ALL feature */
#define  EXF_spare4         (1<<13)  /* reserved for future EXF_ALL feature */
#define  EXF_spare5         (1<<14)  /* reserved for future EXF_ALL feature */
#define  EXF_spare6         (1<<15)  /* reserved for future EXF_ALL feature */

#define  EXF_ALL			(0xFFFF) /* Return all of the information above */


/****************************************************************************/
/* Values for ExamineData->Type for checking what kind of object it describes. 
 * The lower 8 bits form the unique descriptor value, the upper bits are used 
 * as a bitmask. The FSO_TYPE_MASK is used to filter the descriptor value. 
 *
 * NOTE: You MUST specify EXF_TYPE in the EX_DataFields tag for these macros
 *       to work otherwise random values will be returned in the 'Type' field.
 */

#define FSO_TYPE_MASK        0xFF  /* mask for object type descriptors */

#define FSO_TYPE_SOFTLINK    0     /* Object is a softlink */
#define FSO_TYPE_FILE        1     /* Object is a file */
#define FSO_TYPE_DIRECTORY   2     /* Object is a directory */
#define FSO_TYPE_PIPE        3     /* Object is a pipe stream */
#define FSO_TYPE_SOCKET      4     /* Object is a socket stream */

#define FSO_TYPE_INVALID     0xFE  /* Object is unidentifiable or corrupt,
                                      you SHOULD never actually see this,
                                      but the value is reserved here for use 
                                      by the filesystem for internal errors. */


#define FSOF_LINK           (1<<8) /* Bit set for FFS style hard & soft links */
#define FSOF_ALTLINK        (1<<9) /* Reserved bit for alternate style links, */
                                   /* - these are currently unimplemented. */


/****************************************************************************/
/* Macros for identifying what kind of basic object an ExamineData 'Type' describes. 
 * NOTE: These macros mask out the link attribute bits, so check for links first.
 */

/* Standard objects */
#define EXD_IS_FILE(exd)        (((exd)->Type & FSO_TYPE_MASK) == FSO_TYPE_FILE)
#define EXD_IS_DIRECTORY(exd)   (((exd)->Type & FSO_TYPE_MASK) == FSO_TYPE_DIRECTORY)
#define EXD_IS_PIPE(exd)        (((exd)->Type & FSO_TYPE_MASK) == FSO_TYPE_PIPE)
#define EXD_IS_SOCKET(exd)      (((exd)->Type & FSO_TYPE_MASK) == FSO_TYPE_SOCKET)


/*
 * FFS style compatible hard and soft links always set the FSOF_LINK bit.
 * Future Non-FFS style compatible links shall use the FSOF_ALTLINK bit.
 */

/* Generic macro for identifying ALL types of linked objects. (including softlinks) */
#define EXD_IS_LINK(exd)          ((exd)->Type & (FSOF_LINK|FSOF_ALTLINK))

/* Specific macro for identifying a softlink. */
#define EXD_IS_SOFTLINK(exd)      ((exd)->Type == (FSOF_LINK|FSO_TYPE_SOFTLINK))



/****************************************************************************
 * Information relating to the ExamineData; OwnerUID and OwnerGID members.
 *  
 * AmigaDOS permissions are NOT the same as the standard UNIX values.
 * For the DOS/Filesystem UID/GID values;
 *
 *     [nobody/nogroup] = 0       (instead of 65534),
 *     [root/wheel]     = 65535   (instead of 0).
 *
 * These only apply to DOS and the Filesystems, but the C and usergroup
 * libraries use the standard UNIX permissions, where;
 *
 *     [nobody/nogroup] = 65534. 
 *     [root/wheel]     = 0.
 *
 * The AmigaOS C libraries convert between these two protocols.
 */

#define  DOS_OWNER_ROOT     65535
#define  DOS_OWNER_NONE     0


/****************************************************************************
 * Definitions for the ExamineData "Protection" bitfield.
 *
 * EXDB are bit definitions, EXDF are field definitions.
 * Regular RWED bits (0...3) are 0 == allowed. [ie: Negated function]
 *
 * NOTE: GRP and OTR RWED permissions are 0 == not allowed!
 * Group and Other permissions are not directly handled by the filesystem, only stored.
 *
 */

/****************************************************************************/

#define EXDB_OTR_READ         15    /* Other: file is readable */
#define EXDB_OTR_WRITE        14    /* Other: file is writable */
#define EXDB_OTR_EXECUTE      13    /* Other: file is executable */
#define EXDB_OTR_DELETE       12    /* Other: file is deleted */
#define EXDB_GRP_READ         11    /* Group: file is readable */
#define EXDB_GRP_WRITE        10    /* Group: file is writable */
#define EXDB_GRP_EXECUTE       9    /* Group: file is executable */
#define EXDB_GRP_DELETE        8    /* Group: file is deleteable */

#define EXDB_HOLD              7    /* (H) hold loaded program in cli resident list (v50)*/ 
#define EXDB_SCRIPT            6    /* (S) program is a script (execute) file */
#define EXDB_PURE              5    /* (P) program is reentrant and reexecutable */
#define EXDB_ARCHIVE           4    /* (A) cleared whenever file is changed */
#define EXDB_NO_READ           3    /* (R) NOT readable, */
#define EXDB_NO_WRITE          2    /* (W) NOT writable, */
#define EXDB_NO_EXECUTE        1    /* (E) NOT executable, Used by Shell only */
#define EXDB_NO_DELETE         0    /* (D) NOT deletable, */

/*
**  Note that when bits 0,1,2,3  are set, this represent a negated function. 
**  Early contrarily named definitions have been moved to dos/obsolete.h
*/

/****************************************************************************/
/* Masks for above bits */

#define EXDF_OTR_READ         (1L<<EXDB_OTR_READ)
#define EXDF_OTR_WRITE        (1L<<EXDB_OTR_WRITE)
#define EXDF_OTR_EXECUTE      (1L<<EXDB_OTR_EXECUTE)
#define EXDF_OTR_DELETE       (1L<<EXDB_OTR_DELETE)
#define EXDF_GRP_READ         (1L<<EXDB_GRP_READ)
#define EXDF_GRP_WRITE        (1L<<EXDB_GRP_WRITE)
#define EXDF_GRP_EXECUTE      (1L<<EXDB_GRP_EXECUTE)
#define EXDF_GRP_DELETE       (1L<<EXDB_GRP_DELETE)

#define EXDF_HOLD             (1L<<EXDB_HOLD)
#define EXDF_SCRIPT           (1L<<EXDB_SCRIPT)
#define EXDF_PURE             (1L<<EXDB_PURE)
#define EXDF_ARCHIVE          (1L<<EXDB_ARCHIVE)
#define EXDF_NO_READ          (1L<<EXDB_NO_READ)
#define EXDF_NO_WRITE         (1L<<EXDB_NO_WRITE)
#define EXDF_NO_EXECUTE       (1L<<EXDB_NO_EXECUTE)
#define EXDF_NO_DELETE        (1L<<EXDB_NO_DELETE)

/*
 * The following define is the default protection mask value for new objects.
 * This equates to the attributes; R,W,E,D
 */
#define EXDF_DEFAULT   0
  

/****************************************************************************/
/* Standard semantical maximum length for an error string from fault/printfault. 
   However, most error strings should be kept under 60 characters if possible.
   Don't forget space for the header you pass in. */

#define FAULT_MAX  82


/****************************************************************************/
/* Handy macros for testing if a given task/process pointer is actually
 * an EXEC task or a DOS process.   V52.
 */

#define IS_PROCESS(x)     (NT_PROCESS==((struct Task *)x)->tc_Node.ln_Type)
#define IS_TASK(x)        (NT_TASK==((struct Task *)x)->tc_Node.ln_Type)


/****************************************************************************/
/* For the old Info() function, the structure must be on a 4 byte boundary.
 * The new V51+ GetDiskInfo() doesn't require special structure alignment. 
 * Just use AllocDosObject(DOS_INFO,0); to get the right one every time.
 */

struct InfoData
{
    int32  id_NumSoftErrors;    /* number of soft errors on disk */
    int32  id_UnitNumber;       /* Which unit disk is (was) mounted on */
    int32  id_DiskState;        /* See defines below */
    uint32 id_NumBlocks;        /* Number of blocks on disk */
    uint32 id_NumBlocksUsed;    /* Number of block in use */
    uint32 id_BytesPerBlock;    /* The size in bytes of a "block" */
    int32  id_DiskType;         /* Disk Type code */
    BPTR   id_VolumeNode;       /* BCPL pointer to volume node (see DosList) */
    int32  id_InUse;            /* Flag or use count, zero if not in use */
}; 
/* InfoData (ID stands for InfoData) */


/* Disk states */
#define ID_DISKSTATE_WRITE_PROTECTED  80   /* Disk is write protected */
#define ID_DISKSTATE_VALIDATING       81   /* Disk is currently being validated */
#define ID_DISKSTATE_VALIDATED        82   /* Disk is consistent and writeable */


/* Disk types */
#define ID_NO_DISK_PRESENT    (-1)
#define ID_UNREADABLE_DISK    (0x42414400L)    /* 'BAD\0'  */
#define ID_BUSY_DISK          (0x42555359L)    /* 'BUSY'   */
#define ID_DOS_DISK           (0x444F5300L)    /* 'DOS\0'  */
#define ID_FFS_DISK           (0x444F5301L)    /* 'DOS\1'  */
#define ID_INTER_DOS_DISK     (0x444F5302L)    /* 'DOS\2'  */
#define ID_INTER_FFS_DISK     (0x444F5303L)    /* 'DOS\3'  */
#define ID_FASTDIR_DOS_DISK   (0x444F5304L)    /* 'DOS\4'  */
#define ID_FASTDIR_FFS_DISK   (0x444F5305L)    /* 'DOS\5'  */
#define ID_LONGNAME_DOS_DISK  (0x444F5306L)    /* 'DOS\6'  */
#define ID_LONGNAME_FFS_DISK  (0x444F5307L)    /* 'DOS\7'  */

#define ID_VP255_DOS_DISK     (0x444F5309L)    /* 'DOS\9'  */
/*  This V53 dostype is used for newer vector-port based filesystems
**  that support 64 bit file sizes with up to 255 byte long name and
**  comment strings.
**	See MAX_VP_FILENAME and MAX_VP_COMMENT defines above when
**  allocating string buffers for this dostype.
*/

#define ID_NOT_REALLY_DOS     (0x4E444F53L)    /* 'NDOS'  */
#define ID_KICKSTART_DISK     (0x4B49434BL)    /* 'KICK'  */
#define ID_MSDOS_DISK         (0x4d534400L)    /* 'MSD\0' */

/* Special 'disk type' signatures returned by con-handler */
#define ID_CON                (0x434F4E00L)    /* 'CON\0' */
#define ID_RAWCON             (0x52415700L)    /* 'RAW\0' */


/****************************************************************************/
/* Bit numbers that signal that a user has issued a break */

#define SIGBREAKB_CTRL_C 12
#define SIGBREAKB_CTRL_D 13
#define SIGBREAKB_CTRL_E 14
#define SIGBREAKB_CTRL_F 15

/*
**  Bit fields that signal that a user has issued a break,
**  See IDOS->CheckSignal() for examples. 
*/
#define SIGBREAKF_CTRL_C (1L<<SIGBREAKB_CTRL_C)
#define SIGBREAKF_CTRL_D (1L<<SIGBREAKB_CTRL_D)
#define SIGBREAKF_CTRL_E (1L<<SIGBREAKB_CTRL_E)
#define SIGBREAKF_CTRL_F (1L<<SIGBREAKB_CTRL_F)

/****************************************************************************/
/* Values returned by SameLock() */

#define LOCK_DIFFERENT     -1
#define LOCK_SAME           0
#define LOCK_SAME_VOLUME    1    /* locks are on same volume */

/****************************************************************************/
/* Values returned by SameFH() */

#define FH_DIFFERENT     -1
#define FH_SAME           0
#define FH_SAME_VOLUME    1    /* filehandles are on same volume */

/****************************************************************************/
/* types for ChangeMode() */

#define CHANGE_LOCK    0
#define CHANGE_FH      1

/****************************************************************************/
/* Mode parameter for DevNameFromLock() and DevNameFromFH() */

#define DN_DEVICEONLY  0
#define DN_FULLPATH    1

/****************************************************************************/
/* The following is used by the new V51 tag;  NP_NotifyOnDeathMessage       */
/* See CreateNewProc() autodoc for more details.                            */

struct DeathMessage
{
    struct Message dm_Msg;    /* Embedded exec message structure.           */
    int32  dm_ReturnCode;     /* Primary process return code, set by DOS    */
    int32  dm_Result2;        /* The value returned from IoErr(),set by DOS */
};

/****************************************************************************/
/* The following is the data structure to be used for FReadLine()
 * which is allocated by the AllocDosObject() function ONLY.
 */

struct FReadLineData
{
    STRPTR   frld_Line;           /* NUL-terminated string */
    uint32   frld_LineLength;     /* what strlen(frld_Line) would return */
    uint32   frld_DosPrivate[8];  /* -- internal dos.library use only */
};

/****************************************************************************/
/* Mode flags for GetProcSegList()   (v51.88) */

#define GPSLF_CLI    (1<<1)
#define GPSLF_SEG    (1<<2)
#define GPSLF_ENT    (1<<3)
#define GPSLF_RUN    (1<<4)

/****************************************************************************/
/* Values for MakeLink() - DO NOT use random values, new link types will
 * be added in the future that require other values to be used. 
 */

#define LINK_HARD      0
#define LINK_SOFT      1

/****************************************************************************/
/* values returned by ReadItem (and ReadLineItem V50) */

#define ITEM_EQUAL    -2    /* "=" Symbol */
#define ITEM_ERROR    -1    /* error */
#define ITEM_NOTHING   0    /* *N, ;, endstreamch */
#define ITEM_UNQUOTED  1    /* unquoted item */
#define ITEM_QUOTED    2    /* quoted item */

/****************************************************************************/
/* types for AllocDosObject/FreeDosObject */

#define DOS_FILEHANDLE      0   /* Very few people should use this */
#define DOS_EXALLCONTROL    1   /* Must be used to allocate this! - Obsolete struct */
#define DOS_FIB             2   /* may be useful - not really, it's obsolete now */
#define DOS_STDPKT          3   /* for doing packet-level I/O */
#define DOS_CLI             4   /* for shell-writers, etc */
#define DOS_RDARGS          5   /* for ReadArgs if you pass it in */
#define DOS_ANCHORPATH      6   /* for MatchFirst/MatchNext (V50) */
#define DOS_INFODATA        7   /* for Info() aligned InfoData (V50) */
#define DOS_NOTIFYREQUEST   8   /* for Notify request functions (V51)  */
#define DOS_LOCK            9   /* for filesystem lock allocations (V51) */
#define DOS_FREADLINEDATA  10   /* for FReadLine() structure. (V51) */
#define DOS_EXAMINEDATA    11   /* for filesystem ExamineData allocations.(V51) */ 
#define DOS_DOSLIST        12   /* for struct DosList allocations. (V52.16) */
#define DOS_VOLUMELIST     13   /* for struct list with attached nodes. (V53.52) */
#define DOS_FSVECTORPORT   14   /* for filesystem vector port creation. (53.80) */


/****************************************************************************/

#ifdef __GNUC__
   #ifdef __PPC__
    #pragma pack()
   #endif
#elif defined(__VBCC__)
   #pragma default-align
#endif

/****************************************************************************/

#ifdef __cplusplus
}
#ifdef __USE_AMIGAOS_NAMESPACE__
}
#endif
#endif

/****************************************************************************/

#endif /* DOS_DOS_H */
